#!/bin/sh

TAR=tar
DIR=$(basename $(pwd))
DIR_ROOT=$(echo ${DIR} | sed 's/weechat-scripts-/weechat-scripts_/')

${TAR} \
    -czvf ../${DIR_ROOT}.orig.tar.gz ../${DIR} \
    --exclude=".git" \
    --exclude "debian" \
    --transform 's,^\.\./,,'
